import * as path from 'path';
import * as ts from 'typescript';
import merge from 'deepmerge';

const deepMerge = (a, b) => merge(a, b, {
    arrayMerge: (dest, src) => [...dest, ...src],
});
const readConfigFile = (configPath) => {
    let config = ts.readConfigFile(configPath, ts.sys.readFile).config;
    if (config.extends) {
        const extendsPath = ts.findConfigFile(path.resolve(path.dirname(configPath), config.extends), ts.sys.fileExists);
        if (extendsPath && extendsPath !== configPath) {
            config = deepMerge(readConfigFile(extendsPath), config);
        }
    }
    return config;
};
/**
 * reads any typescript configuration files for a given file, including the extends references
 * @param filePath the full file path to the file
 * @param defaultConfig optional default configuration
 * @param keepJson set to tru to avoid converting options
 * @returns the typescript configuration for the file, or undefined if this is not a typescript file
 */
const getTypescriptConfig = (filePath, defaultConfig, keepJson) => {
    var _a;
    const ext = ((_a = filePath.split('.').pop()) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || 'js';
    if (['ts', 'tsx'].indexOf(ext) !== -1) {
        let config = defaultConfig || {};
        const configPath = ts.findConfigFile(path.dirname(filePath), ts.sys.fileExists);
        if (configPath) {
            const fileConfig = readConfigFile(configPath);
            if (fileConfig === null || fileConfig === void 0 ? void 0 : fileConfig.compilerOptions) {
                config = deepMerge(config, fileConfig === null || fileConfig === void 0 ? void 0 : fileConfig.compilerOptions);
            }
        }
        if (keepJson) {
            return config;
        }
        const { options } = ts.convertCompilerOptionsFromJson(config, '.');
        return options;
    }
    return undefined;
};

export { getTypescriptConfig };
