'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var ts = require('typescript');
var merge = require('deepmerge');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      }
    });
  }
  n['default'] = e;
  return Object.freeze(n);
}

var path__namespace = /*#__PURE__*/_interopNamespace(path);
var ts__namespace = /*#__PURE__*/_interopNamespace(ts);
var merge__default = /*#__PURE__*/_interopDefaultLegacy(merge);

const deepMerge = (a, b) => merge__default['default'](a, b, {
    arrayMerge: (dest, src) => [...dest, ...src],
});
const readConfigFile = (configPath) => {
    let config = ts__namespace.readConfigFile(configPath, ts__namespace.sys.readFile).config;
    if (config.extends) {
        const extendsPath = ts__namespace.findConfigFile(path__namespace.resolve(path__namespace.dirname(configPath), config.extends), ts__namespace.sys.fileExists);
        if (extendsPath && extendsPath !== configPath) {
            config = deepMerge(readConfigFile(extendsPath), config);
        }
    }
    return config;
};
/**
 * reads any typescript configuration files for a given file, including the extends references
 * @param filePath the full file path to the file
 * @param defaultConfig optional default configuration
 * @param keepJson set to tru to avoid converting options
 * @returns the typescript configuration for the file, or undefined if this is not a typescript file
 */
const getTypescriptConfig = (filePath, defaultConfig, keepJson) => {
    var _a;
    const ext = ((_a = filePath.split('.').pop()) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || 'js';
    if (['ts', 'tsx'].indexOf(ext) !== -1) {
        let config = defaultConfig || {};
        const configPath = ts__namespace.findConfigFile(path__namespace.dirname(filePath), ts__namespace.sys.fileExists);
        if (configPath) {
            const fileConfig = readConfigFile(configPath);
            if (fileConfig === null || fileConfig === void 0 ? void 0 : fileConfig.compilerOptions) {
                config = deepMerge(config, fileConfig === null || fileConfig === void 0 ? void 0 : fileConfig.compilerOptions);
            }
        }
        if (keepJson) {
            return config;
        }
        const { options } = ts__namespace.convertCompilerOptionsFromJson(config, '.');
        return options;
    }
    return undefined;
};

exports.getTypescriptConfig = getTypescriptConfig;
